/* OAIHTMLProcessor.m created by luke on Wed 24-Feb-1999 */

#import "OAIHTMLProcessor.h"

@implementation OAIHTMLProcessor

static unsigned int imageAltAttributeIndex;
static unsigned int metaNameAttributeIndex;
static unsigned int metaContentAttributeIndex;

static CSBitmap nonLetterBitmap = 0;

+ (void)initialize;
{
    NSCharacterSet *set;

    [super initialize];
    if (nonLetterBitmap)
        return;
    
    set = [[NSCharacterSet letterCharacterSet] invertedSet];
    nonLetterBitmap = bitmapForCharacterSetDoRetain(set, YES);
}

+ (void)didLoad;
{
    OWSGMLMethods *methods;
    OWSGMLTagType *imageTagType;
    OWSGMLTagType *metaTagType;
    OWSGMLDTD *dtd;

    [self registerProcessorClass:self fromContentType:[OWContentType contentTypeForString:@"objectstream/sgml"] toContentType:[self destinationType] cost:1.0];

    dtd = [self dtd];

    imageTagType = [dtd tagTypeNamed:@"img"];
    metaTagType = [dtd tagTypeNamed:@"meta"];

    imageAltAttributeIndex = [imageTagType addAttributeNamed:@"alt"];
    metaNameAttributeIndex = [metaTagType addAttributeNamed:@"name"];
    metaContentAttributeIndex = [metaTagType addAttributeNamed:@"content"];

    methods = [self sgmlMethods];
    [methods registerMethod:@"Image" forTagName:@"img"];
    [methods registerMethod:@"Meta" forTagName:@"meta"];
}

+ (OWSGMLDTD *)dtd;
{
    static OWSGMLDTD *htmlDTD = nil;
    OWContentType *html;
    OWContentType *sgml;

    if (htmlDTD)
        return htmlDTD;

    html = [OWContentType contentTypeForString:@"text/html"];

    sgml = [OWContentType contentTypeForString:@"objectstream/sgml"];
    [OWHTMLToSGMLObjects registerProcessorClass:[OWHTMLToSGMLObjects class] fromContentType:html toContentType:sgml cost:1.0];

    htmlDTD = [OWSGMLDTD registeredDTDForSourceContentType:html
                                    destinationContentType:sgml];
    return htmlDTD;
}

+ (OWContentType *)destinationType;
{
    static OWContentType *type = nil;

    if (!type)
        type = [OWContentType contentTypeForString:@"objectstream/htmltokens"];
    return type;
}

- (id)initWithPipeline:(OWPipeline *)aPipeline;
{
    if (!(self = [super initWithPipeline:aPipeline]))
        return nil;

    tokenGroupScanner = [[OFStringScanner alloc] init];

    objectStream = [[OWObjectStream alloc] init];
    [objectStream setContentType:[isa destinationType]];

    [aPipeline addContent:objectStream];
    [aPipeline startProcessingContent];

    return self;
}

- (void)dealloc;
{
    [tokenGroupScanner release];
    [objectStream release];

    [super dealloc];
}

- (void)processImageTag:(OWSGMLTag *)tag;
{
    NSString *string;

    if ((string = sgmlTagValueForAttributeAtIndex(tag, imageAltAttributeIndex)))
        [self processTokenGroup:string];
}

- (void)processMetaTag:(OWSGMLTag *)tag;
{
    NSString *string;
    
    if ((string = sgmlTagValueForAttributeAtIndex(tag, metaNameAttributeIndex)))
        [self processTokenGroup:string];
    if ((string = sgmlTagValueForAttributeAtIndex(tag, metaContentAttributeIndex)))
        [self processTokenGroup:string];
}

- (void)processCData:(NSString *)cData;
{
    [self processTokenGroup:cData];
}

- (void)processTokenGroup:(NSString *)tokenString;
{
    [tokenGroupScanner fetchMoreDataFromString:tokenString];

    while (scannerHasData(tokenGroupScanner)) {
        NSString *token;

        scannerScanUpToCharacterNotInCSBitmap(tokenGroupScanner, nonLetterBitmap);
        if ((token = [tokenGroupScanner readFullTokenWithDelimiterCSBitmap:nonLetterBitmap forceLowercase:NO]))
            [objectStream writeObject:token];
    }
}

// OWProcessor subclassed methods

- (void)processEnd;
{
    [objectStream dataEnd];
    [super processEnd];
}

- (void)processAbort;
{
    [objectStream dataAbort];
    [super processAbort];
}

@end
