// Copyright 1998 Omni Development, Inc.  All rights reserved.

#import "OAIHTMLTokenizer.h"

#import <Foundation/Foundation.h>
#import <OmniBase/OmniBase.h>
#import <OmniFoundation/OmniFoundation.h>
#import "OAIHTMLProcessor.h"

@implementation OAIHTMLTokenizer

- initWithScanner:(OFStringScanner *)aScanner;
{
    if (!(self = [super init]))
        return nil;

    targetScanner = [aScanner retain];
    isDebug = [[NSUserDefaults standardUserDefaults] boolForKey:@"OAIDebugHTMLTokenizer"];

    hasStartedPipeline = NO;
    
    return self;
}

- (void)dealloc;
{
    [OWPipeline cancelTarget:self];
    
    [targetScanner release];
    [targetString release];
    [tokenCursor release];
    [tokenStream release];
    [processingLock release];
    
    [super dealloc];
}

- (id <OAITokenizerInstance>)tokenizerForScanner:(OFStringScanner *)aScanner;
{
    return [[[isa alloc] initWithScanner:aScanner] autorelease];
}

- (NSString *)nextToken;
{
    NSString *nextToken;

    if (!hasStartedPipeline) {
        NSData *stringData;
        OWFileDataStream *fileDataStream;

        targetString = [[targetScanner string] retain];
        if (!(stringData = [targetString dataUsingEncoding:NSNonLossyASCIIStringEncoding]))
            NSLog(@"Couldn't create string data from scanner string.");
        fileDataStream = [[OWFileDataStream alloc] initWithData:stringData];
        [fileDataStream setContentTypeString:@"text/html"];

        processingLock = [[NSLock alloc] init];
        [processingLock lock];
        [OWWebPipeline startPipelineWithContent:fileDataStream target:self];
        [processingLock lock];
        [fileDataStream release];
            
        hasStartedPipeline = YES;
    }

    if (isDebug) {
        nextToken = [tokenCursor readObject];
        NSLog(@"Next token: %@", nextToken);
        return nextToken;
    }
    
    return [tokenCursor readObject];
}

// OWTargetProtocolConformance

- (OWContentType *)targetContentType;
{
   return [OAIHTMLProcessor destinationType];
}

- (OWTargetContentDisposition)pipeline:(OWPipeline *)aPipeline hasContent:(id <OWContent>)someContent;
{
    tokenStream = [[aPipeline lastContent] retain];
    tokenCursor = [[OWObjectStreamCursor alloc] initForObjectStream:tokenStream];

    [processingLock unlock];
    return OWTargetContentDisposition_ContentAccepted;
}

- (OWContentInfo *)parentContentInfo;
{
    return [OWContentInfo headerContentInfoWithName:@"HTML Tokenizing"];
}

- (NSString *)targetTypeFormatString;
{
    return nil;
}

@end
